 //-------------------------------------------------------
//EXTERN GLOBAL VARIABLES
//-------------------------------------------------------
//Matrices
float4x4 matW : WORLD;
float4x4 matV : VIEW;
float4x4 matP : PROJECTION;
float4x4 matWV;
float4x4 matWVP;

//Inverted matrices (as in NV demo)
float4x4 matInvTrW;
float4x4 matInvV;
float4x4 matInvP;
float4x4 matInvVP;

//Light parametres
float3 l_colAmbient;// Light color
float3 l_dir; 		// Light direction (view space)
float3 l_pos; 		// Light position (view space)	
float  l_radius;	// Light radius (for point light)

//OTHER
float ViewAspect;

//float3 View;

//-------------------------------------------------------
//TEXTURES
//-------------------------------------------------------
// Texture
texture m_TexAlfa;
texture m_TexBump;
texture m_TexDiffuse;
texture m_TexSpecular;

//BEG - Samplers
		sampler AlfaSampler = sampler_state
		{
		Texture = (m_TexAlfa);
		MipFilter = LINEAR;
		MinFilter = LINEAR;
		MagFilter = LINEAR;
		};
		sampler BumpSampler = sampler_state
		{
		Texture = (m_TexBump);
		MipFilter = LINEAR;
		MinFilter = LINEAR;
		MagFilter = LINEAR;
		};
		sampler DiffSampler = sampler_state
		{
		Texture = (m_TexDiffuse);
		MipFilter = LINEAR;
		MinFilter = LINEAR;
		MagFilter = LINEAR;
		};
		sampler SpecSampler = sampler_state
		{
		Texture = (m_TexSpecular);
		MipFilter = LINEAR;
		MinFilter = LINEAR;
		MagFilter = LINEAR;
		};
//End - Samplers

//-------------------------------------------------------
//INTERNAL VARIABLES
//-------------------------------------------------------
//Vertex struct for in data
struct VS_IN
{
float4 Pos 		: POSITION;
float3 Norm 	: NORMAL;
float3 Tang 	: TANGENT;
float3 Binorm 	: BINORMAL; 
float2 Tex 		: TEXCOORD0;
};
//Vertex struct for data exchange beetwen VS & PS
struct VS_OUT
{
float4 PosProj 		: POSITION;
float2 Tex  		: TEXCOORD0;
float4 PosView    	: TEXCOORD1;
//float3 ViewDir    	: TEXCOORD3;
float3 WorldNormal	: TEXCOORD4;
float3 WorldTangent	: TEXCOORD5;
float3 WorldBinorm	: TEXCOORD6;
};
//Vertex struct for in data DS
struct VS_INDS
{
float4 Pos 		: POSITION;
float2 Tex 		: TEXCOORD0;
};
//Vertex struct for data exchange beetwen VS & PS DS
struct VS_OUTDS
{
float4 PosProj 		: POSITION;
float2 Tex  		: TEXCOORD0;
float4 PosView    	: TEXCOORD1;
};
//--------------------------------------------------------------------
float3 float_to_color(in float f)
{
   float3 color;
   f *= 256;
   color.x = floor(f);
   f = (f-color.x)*256;
   color.y = floor(f);
   color.z = f-color.y;
   color.xy *= 0.00390625; // *= 1.0/256
   return color;
}
//--------------------------------------------------------------------
float color_to_float(float3 color)
{
   const float3 byte_to_float=float3(1.0,1.0/256,1.0/(256*256));
   return dot(color,byte_to_float);
}
//--------------------------------------------------------------------

//Classic lighting
VS_OUT VSClassic (VS_IN In)
{
VS_OUT Out;
// Transforms Normals and t.c. to View space
Out.WorldNormal		= mul(In.Norm,	(float3x3)matV);
Out.WorldTangent	= mul(In.Tang,	(float3x3)matV);
Out.WorldBinorm		= mul(In.Binorm,(float3x3)matV);
// Position (world space)
Out.PosView = mul(In.Pos, matV);
Out.PosProj = mul(In.Pos, matWVP);
// Texture coords copying
Out.Tex = In.Tex;
return Out;
}


//Deffered Shading Prepare
VS_OUT VS (VS_IN In)
{
VS_OUT Out;
// Transforms Normals and t.c. to View space
Out.WorldNormal		= mul(In.Norm,	(float3x3)matV);
Out.WorldTangent	= mul(In.Tang,	(float3x3)matV);
Out.WorldBinorm		= mul(In.Binorm,(float3x3)matV);
// Position (world space)
Out.PosProj = mul(In.Pos, matWVP);
Out.PosView = mul(In.Pos, matWVP);
// Texture coords copying
Out.Tex = In.Tex;
return Out;
}

//Deffered Shading
VS_OUTDS VSDS (VS_INDS In)
{
VS_OUTDS Out;
// Position
Out.PosProj = mul( float4(In.Pos.xyz , 1.0), matWVP);
Out.PosView = Out.PosProj;
// Texture coords copying
Out.Tex = In.Tex;

return Out;
}

float4 PSPosition (VS_OUT In):COLOR
{
	float4 MRTtexture_vpos;

	MRTtexture_vpos.xyz = float_to_color(In.PosView.z / In.PosView.w);
	MRTtexture_vpos.a = tex2D(AlfaSampler, In.Tex).x;
	return MRTtexture_vpos;
}

float4 PSNormal (VS_OUT In):COLOR
{
	float3 normalMap = tex2D(BumpSampler,In.Tex).xyz-(0.5).xxx;

    float3 Nn = normalize(In.WorldNormal);
    float3 Tn = normalize(In.WorldTangent);
    float3 Bn = normalize(In.WorldBinorm);

    float3 Nb = Nn + (normalMap.x * Tn + normalMap.y * Bn);

	float3 N = normalize(Nb);
	float4 MRTtexture_normal;
	MRTtexture_normal.xyz = N.xyz*0.5+(0.5).xxx;
	MRTtexture_normal.a =  tex2D(AlfaSampler, In.Tex).x;
	return MRTtexture_normal;
}

float4 PSDiffuse (VS_OUT In):COLOR
{
	float4 MRTtexture_diffcol;
	MRTtexture_diffcol.xyz = tex2D(DiffSampler, In.Tex).xyz;
	MRTtexture_diffcol.a =  tex2D(AlfaSampler, In.Tex).x;
	return MRTtexture_diffcol;
}

float4 PSSpecular (VS_OUT In):COLOR
{
	float4 MRTtexture_specular;
	MRTtexture_specular.xyz = tex2D(SpecSampler,In.Tex).xyz;
	MRTtexture_specular.a =  tex2D(AlfaSampler, In.Tex).x;//tex2D(SpecSampler,In.Tex).a;
	return MRTtexture_specular;
}

//Classic point light
float4 PSPointLight (VS_OUT In):COLOR
{
	//Bump computing
	float3 normalMap = (tex2D(BumpSampler,In.Tex).xyz-(0.5).xxx);
	float3 Nn = normalize(In.WorldNormal);
	float3 Tn = normalize(In.WorldTangent);
	float3 Bn = normalize(In.WorldBinorm);
	float3 Nb = Nn + (normalMap.x * Tn + normalMap.y * Bn);
	float3 N = normalize(Nb);

	//Position
	float3 Pos  = In.PosView.xyz;
	
	//Diffuse color
	float3 DiffColor;
	DiffColor.xyz =  (tex2D(DiffSampler, In.Tex)).xyz;
	//Specular
	float3 Mat_Specular_Power = tex2D(SpecSampler,In.Tex).xyz;
	
	//Foley attenuation computing
	float r = distance(Pos, l_pos);
	float Foley = (l_radius/r);

	//View vector
	float3 Vn = normalize(Pos);
	//Light vector	
	float3 Ln = l_pos - Pos;
	//H
	float3 Hn = normalize(Vn + Ln);

	Ln = normalize(Ln);	
	//Specular
	float Spec = pow(saturate(dot(Hn,Nb)),Mat_Specular_Power);
	//Diffuse
	float Diff = saturate(dot(Ln,Nb));
	
	float specContrib = (Diff * Spec);

	//Result
	float4 result;
	result.xyz = l_colAmbient*DiffColor*Diff + l_colAmbient*Mat_Specular_Power.xyz*specContrib;
	result.xyz = result.xyz*Foley;
	result.a = tex2D(AlfaSampler, In.Tex).x;
return result;   
}

//Classic SunLight
float4 PSSunLight (VS_OUT In):COLOR
{
//Bump computing
	float3 normalMap = (tex2D(BumpSampler,In.Tex).xyz-(0.5).xxx);
	float3 Nn = normalize(In.WorldNormal);
	float3 Tn = normalize(In.WorldTangent);
	float3 Bn = normalize(In.WorldBinorm);
	float3 Nb = Nn + (normalMap.x * Tn + normalMap.y * Bn);
	float3 N = normalize(Nb);

	//Position
	float3 Pos  = In.PosView.xyz;
	
	//Diffuse color
	float3 DiffColor;
	DiffColor.xyz =  (tex2D(DiffSampler, In.Tex)).xyz;
	//Specular
	float3 Mat_Specular_Power = tex2D(SpecSampler,In.Tex).xyz;
	
	//View vector
	float3 Vn = normalize(Pos);
	
	//Light vector
	l_dir = mul(float4(l_dir.xyz,1.0),matV);
	float3 Ln = l_pos - Pos;
	//H
	float3 Hn = normalize(Vn + Ln);

	Ln = normalize(Ln);
	//Specular
	float Spec = pow(saturate(dot(Hn,Nb)),Mat_Specular_Power);
	//Diffuse
	float Diff = saturate(dot(Ln,Nb));
	
	float specContrib = (Diff * Spec);
 
	//Result
	float4 result;
	result.xyz = l_colAmbient*DiffColor+l_colAmbient*DiffColor*Diff + l_colAmbient*Mat_Specular_Power.xyz*specContrib;// + reflColor; l_colAmbient
	result.a = tex2D(AlfaSampler, In.Tex).x;
	return result;   
}


//Deffered Shading Sun
float4 PSDSSun (VS_OUTDS In):COLOR
{
	//Loading data from RT textures
	float3 Mat_Specular_Power = tex2D(SpecSampler,In.Tex).xyz;
	float3 DiffColor = tex2D(DiffSampler,In.Tex).xyz;
	float3 n = 2*(tex2D(BumpSampler,In.Tex).xyz-(0.5).xxx);	
	float z = color_to_float (tex2D(AlfaSampler,In.Tex));

	//reconstruct original view-space position (From NV Demo)
	float x = In.PosView.x;
	float y = In.PosView.y;
	float4x4 matVP;
	matVP = mul(matV,matP);
	float4 Pos = mul(float4(x, y, z, 1.0), matInvVP);
	Pos.xyz = Pos.xyz / Pos.www;
	Pos.w = 1.0f;
	
	//Light position to View space (Why inverted? Don`t know? But it must be so!)
	l_pos = mul(float4(l_pos.xyz, 1.0), matInvV);

	//Attenuation (by Folley formula. It`s not optimal? But I like result!)
	float r = distance(l_pos, Pos);
	float Foley = saturate(l_radius/r);

	// light vector
	float3 l = l_pos - Pos.xyz;
	// view vector
    	float3 v = normalize(-Pos.xyz);
	// half vector (here light vector must not be normilized! Why?)
    	float3 h = normalize(v + l);
	
    	// diffuse and specular terms
    	l = normalize(l); //And here light vector must be normalized
	float3 Idiff = saturate(dot(l,n)); //Diffuse part of light
    	float3 Ispec = pow(saturate(dot(h,n)),Mat_Specular_Power); //Specular part of light
	float specContrib = (Idiff * Ispec); //Specular with diffuse?

	//And final color computing
	float4 result;
   	result.xyz = l_colAmbient*Idiff*DiffColor + l_colAmbient*Mat_Specular_Power.xyz*specContrib;
	result.xyz = l_colAmbient*DiffColor + Foley*result.xyz;
	result.a = 1.0;
	//Yeah! I`ve done it!
	return result;
}

//Deffered Shading Point Light
float4 PSDSPoint (VS_OUTDS In):COLOR
{
	//Loading data from RT textures
	float3 Mat_Specular_Power = tex2D(SpecSampler,In.Tex).xyz;
	float3 DiffColor = tex2D(DiffSampler,In.Tex).xyz;
	float3 n = 2*(tex2D(BumpSampler,In.Tex).xyz-(0.5).xxx);	
	float z = color_to_float (tex2D(AlfaSampler,In.Tex));

	//reconstruct original view-space position (From NV Demo)
	float x = In.PosView.x;
	float y = In.PosView.y;
	float4x4 matVP;
	matVP = mul(matV,matP);
	float4 Pos = mul(float4(x, y, z, 1.0), matInvVP);
	Pos.xyz = Pos.xyz / Pos.www;
	Pos.w = 1.0f;
	
	//Light position to View space (Why inverted? Don`t know? But it must be so!)
	l_pos = mul(float4(l_pos.xyz, 1.0), matInvV);

	//Attenuation (by Folley formula. It`s not optimal? But I like result!)
	float r = distance(l_pos, Pos);
	float Foley = (l_radius/r);

	// light vector
	float3 l = l_pos - Pos.xyz;
	// view vector
    float3 v = normalize(-Pos.xyz);
	// half vector (here light vector must not be normilized! Why?)
    float3 h = normalize(v + l);
	
    // diffuse and specular terms
    l = normalize(l); //And here light vector must be normalized
	float3 Idiff = saturate(dot(l,n)); //Diffuse part of light
    float3 Ispec = pow(saturate(dot(h,n)),Mat_Specular_Power); //Specular part of light
	float specContrib = (Idiff * Ispec); //Specular with diffuse?

	//And final color computing
	float4 result;
   	result.xyz = l_colAmbient*Idiff*DiffColor + l_colAmbient*Mat_Specular_Power.xyz*specContrib;
//   	result.xyz = Idiff*DiffColor + Mat_Specular_Power.xyz*specContrib;

	result.xyz = Foley*result.xyz;
	result.a = 1.0;
	//Yeah! I`ve done it!
	return result;
}


technique RenderClassicSun
{
pass P0
{
// Shaders
VertexShader = compile vs_2_0 VSClassic();
PixelShader = compile ps_2_0 PSSunLight();
}
}


technique RenderClassicPoint
{
pass P0
{
// Shaders
VertexShader = compile vs_2_0 VSClassic();
PixelShader = compile ps_2_0 PSPointLight();
}
}

technique RenderDiffuseToTexture
{
pass P0
{
// Shaders
VertexShader = compile vs_2_0 VS();
PixelShader = compile ps_2_0 PSDiffuse();

		ALPHATESTENABLE	= TRUE;
		SRCBLEND		= SRCALPHA;
		DESTBLEND		= INVSRCALPHA;
		ZENABLE			= TRUE;
}
}

Technique RenderSpecularToTexture
{
pass P0
{
// Shaders
VertexShader = compile vs_2_0 VS();
PixelShader = compile ps_2_0 PSSpecular();
		ALPHATESTENABLE	= TRUE;
		SRCBLEND		= SRCALPHA;
		DESTBLEND		= INVSRCALPHA;
		ZENABLE			= TRUE;
}
}

Technique RenderPositionToTexture
{
pass P0
{
// Shaders
VertexShader = compile vs_2_0 VS();
PixelShader = compile ps_2_0 PSPosition();
		ALPHATESTENABLE	= TRUE;
		SRCBLEND		= SRCALPHA;
		DESTBLEND		= INVSRCALPHA;
		ZENABLE			= TRUE;

}
}

Technique RenderNormalToTexture
{
pass P0
{
// Shaders
VertexShader = compile vs_2_0 VS();
PixelShader = compile ps_2_0 PSNormal();
		ALPHATESTENABLE	= TRUE;
		SRCBLEND		= SRCALPHA;
		DESTBLEND		= INVSRCALPHA;
		ZENABLE			= TRUE;

}
}

//Deferred shading technique for Sunlight
Technique RenderDSSun
{
pass P0
{
// Shaders
VertexShader = compile vs_2_0 VSDS();
PixelShader  = compile ps_2_0 PSDSSun();
		ALPHATESTENABLE	= TRUE;
		ALPHABLENDENABLE= TRUE;
		SRCBLEND		= ONE;
		DESTBLEND		= ONE;
		ZENABLE			= FALSE;
}
}

//Deferred shading technique for Pointlight
Technique RenderDSPoint
{
pass P0
{
// Shaders
VertexShader = compile vs_2_0 VSDS();
PixelShader  = compile ps_2_0 PSDSPoint();
		ALPHATESTENABLE	= TRUE;
		ALPHABLENDENABLE= TRUE;
		SRCBLEND		= ONE;
		DESTBLEND		= ONE;
		ZENABLE			= FALSE;

}
}