/************************************************************************/
/* 	Test application for Deferred Shading technology and it`s compatibility with current
/*	Chassis2 render technologies.	
/*	$id$
/*	File	: 					CMeshX.cpp
/*	Version	:					1.0
/*	Descript:					The mesh working class: load, render
/*	Autor	:					Morock
/*	Created :					2/20/2007
/************************************************************************/

#include "CMeshX.h"

#include <string.h>


CMeshX::CMeshX()
{
	TheMeshIsCorrect = FALSE;
	pMesh = NULL;
	dwNumMaterials=0;
	pMaterials=NULL;
}

CMeshX::~CMeshX()
{
	
	for (DWORD i=0; i<dwNumMaterials; i++) //textures
	{
		SAFE_RELEASE(pMaterials[i].pTexAlfa);
		SAFE_RELEASE(pMaterials[i].pTexBump);
		SAFE_RELEASE(pMaterials[i].pTexDiffuse);
		SAFE_RELEASE(pMaterials[i].pTexSpecular);
	}
	SAFE_DELETE_ARRAY(pMaterials);	//array of materials
	SAFE_RELEASE(pMesh);		//current mesh
	dwNumMaterials = 0;			//number of materials in mesh	
	TheMeshIsCorrect = FALSE;
}

HRESULT CMeshX::Load_Mesh (LPDIRECT3DDEVICE9 pd3dDevice, WCHAR *pMeshFileName)
{
	HRESULT hr;
	LPD3DXBUFFER pD3DXMtrlBuffer=NULL;

	//Loading mesh
	int itmp = 0;
	if (0 == _waccess_s(pMeshFileName,itmp))
	{
		hr =  D3DXLoadMeshFromX( pMeshFileName, D3DXMESH_MANAGED, 
			pd3dDevice, NULL, 
			&pD3DXMtrlBuffer, NULL, &dwNumMaterials, 
			&pMesh);
		if ( FAILED(hr) )
			return hr; //saddly :-(
	}
	else
	{
		return D3DERR_NOTFOUND;
	}
	//Bump data computing
	hr = D3DXComputeNormals(pMesh,NULL);
	hr = D3DXComputeTangent(pMesh,0,0,0,0,NULL);

	//Temp material pointer
	D3DXMATERIAL* d3dxMaterials = (D3DXMATERIAL*)pD3DXMtrlBuffer->GetBufferPointer();
	
	//Allocating memory for materials array
	pMaterials = new CMeshXMaterial[dwNumMaterials];
	if( pMaterials == NULL )
		return E_OUTOFMEMORY;
	
	for( DWORD i=0; i<dwNumMaterials; i++ )
	{
		//Something says to me, that CreateTexture will do the memory allocation by itself? Or not? Yes!
		pMaterials[i].pTexAlfa		= NULL;
		pMaterials[i].pTexBump		= NULL;
		pMaterials[i].pTexDiffuse	= NULL;
		pMaterials[i].pTexSpecular	= NULL;
		//if texture filename present
		if( d3dxMaterials[i].pTextureFilename != NULL)
		{
			//Lets set the file path from wide (C:\\\bla\\\blabla\\\bla.txt) as it do x file exporter,
			//to local "Textures\\"
			WCHAR pTempFileName[MAX_PATH];
			WCHAR pTextureFilename[MAX_PATH];
			WCHAR pdest[MAX_PATH], pFName[MAX_PATH], pFExt[MAX_PATH];
			size_t  itmp_size;
			//f*ck`n D3DXMATERIAL * d3dxMaterials is not UNICODE!!!
			mbstowcs_s(
				&itmp_size,			//size_t *pConvertedChars,
				pTextureFilename,
				sizeof(pTextureFilename),		//size_t sizeInWords,
				d3dxMaterials[i].pTextureFilename,
				(size_t)MAX_PATH 
				);
			_wsplitpath_s(
							pTextureFilename,
							pdest,//wchar_t * drive,
							itmp_size,//size_t driveSizeInCharacters,
							pdest,//wchar_t *dir,
							itmp_size,//size_t dirSizeInCharacters,
							pFName,//wchar_t * fname,
							itmp_size,//size_t nameSizeInCharacters,
							pFExt,//wchar_t * ext,
							itmp_size//size_t extSizeInCharacters
						);
			wcscpy_s(pTempFileName,sizeof(L"textures\\"),L"textures\\");
			wcscat_s(pTempFileName,pFName);
			wcscat_s(pTempFileName,pFExt);
			// Create the diffuse texture if file exist
			if (0 == _waccess_s(pTempFileName,itmp))
			{
					hr = D3DXCreateTextureFromFileW(pd3dDevice, 
													pTempFileName, 
													&pMaterials[i].pTexDiffuse); 
					if (FAILED(hr))
					{
						return hr;
					}
			}
			else
			{	
				//if file is not exist load default white texture
				hr = D3DXCreateTextureFromFileW(pd3dDevice,
					L"textures\\none.bmp",
					&pMaterials[i].pTexDiffuse); 
				if (FAILED(hr))
				{
					return hr;
				}
			}
			wcscpy_s(pTempFileName,sizeof(L"textures\\"),L"textures\\");
			wcscat_s(pTempFileName,pFName);
			wcscat_s(pTempFileName,L"_a");
			wcscat_s(pTempFileName,pFExt);
			// Create the alfa texture if file exist
			if (0 == _waccess_s(pTempFileName,itmp))
			{
				hr = D3DXCreateTextureFromFileW(pd3dDevice, 
					pTempFileName, 
					&pMaterials[i].pTexAlfa); 
				if (FAILED(hr))
				{
					return hr;
				}
			}
			else
			{
				hr = D3DXCreateTextureFromFileW(pd3dDevice,
					L"textures\\none.bmp",
					&pMaterials[i].pTexAlfa); 
				if (FAILED(hr))
				{
					return hr;
				}
			}
			wcscpy_s(pTempFileName,sizeof(L"textures\\"),L"textures\\");
			wcscat_s(pTempFileName,pFName);
			wcscat_s(pTempFileName,L"_b");
			wcscat_s(pTempFileName,pFExt);
			// Create the bump texture if file exist
			if (0 == _waccess_s(pTempFileName,itmp))
			{
				hr = D3DXCreateTextureFromFileW(pd3dDevice, 
					pTempFileName, 
					&pMaterials[i].pTexBump); 
				if (FAILED(hr))
				{
					return hr;
				}
			}
			else
			{
				hr = D3DXCreateTextureFromFileW(pd3dDevice,
					L"textures\\none.bmp",
					&pMaterials[i].pTexBump); 
				if (FAILED(hr))
				{
					return hr;
				}
			}
			wcscpy_s(pTempFileName,sizeof(L"textures\\"),L"textures\\");
			wcscat_s(pTempFileName,pFName);
			wcscat_s(pTempFileName,L"_s");
			wcscat_s(pTempFileName,pFExt);
			// Create the specular texture if file exist
			if (0 == _waccess_s(pTempFileName,itmp))
			{
				hr = D3DXCreateTextureFromFileW(pd3dDevice, 
					pTempFileName, 
					&pMaterials[i].pTexSpecular); 
				if (FAILED(hr))
				{
					return hr;
				}
			}
			else
			{
				hr = D3DXCreateTextureFromFileW(pd3dDevice,
					L"textures\\none.bmp",
					&pMaterials[i].pTexSpecular); 
				if (FAILED(hr))
				{
					return hr;
				}
			}
		}//End if texture filename present
	}//End material loop
	SAFE_RELEASE(pD3DXMtrlBuffer);
	TheMeshIsCorrect = TRUE;
	return S_OK;
}

HRESULT CMeshX::Draw_Mesh(LPDIRECT3DDEVICE9 pd3dDevice,  ID3DXEffect * pEffect, D3DXHANDLE hShadTech)
{
	HRESULT hr;

	if (!TheMeshIsCorrect)
	{
		return E_FAIL;
	}
	//Set Render States
	pd3dDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
	pd3dDevice->SetRenderState(D3DRS_ALPHAREF, (DWORD)0x00000001);
	pd3dDevice->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL);
	pd3dDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	pd3dDevice->SetRenderState(D3DRS_AMBIENT, 0x00202020);
	pd3dDevice->SetRenderState(D3DRS_SPECULARENABLE, FALSE);
	pd3dDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);	
	pd3dDevice->SetRenderState(D3DRS_ZENABLE, TRUE);

	//Set Shader variables
	//Matrices
	hr = pEffect->SetMatrix("matWVP",&matWVP);
	hr = pEffect->SetMatrix("matW",&matW);
	hr = pEffect->SetMatrix("matV",&matV);
	hr = pEffect->SetMatrix("matP",&matP);
		
	//Set Shader Properties
	hr = pEffect->SetTechnique(hShadTech);
	
	//Draw mesh with shader
	UINT iPasses;	
	hr = pEffect->Begin(&iPasses,0); 
	for( DWORD i=0; i<dwNumMaterials; i++ )
	{
		//Material textures
		hr = pEffect->SetTexture("m_TexAlfa",pMaterials[i].pTexAlfa);//texture m_TexAlfa;
		hr = pEffect->SetTexture("m_TexBump",pMaterials[i].pTexBump);//texture m_TexBump;
		hr = pEffect->SetTexture("m_TexDiffuse",pMaterials[i].pTexDiffuse);//texture m_TexDiffuse;
		hr = pEffect->SetTexture("m_TexSpecular",pMaterials[i].pTexSpecular);//texture m_TexSpecular;

		hr = pEffect->BeginPass(0);
				// Draw the mesh subset
				hr = pMesh->DrawSubset(i);
				if (FAILED(hr))
				{
					return hr;
				}
		hr = pEffect->EndPass();
	}
	hr = pEffect->End();
	return S_OK;
}

HRESULT CMeshX::Update_Mesh(LPDIRECT3DDEVICE9 pd3dDevice, D3DXVECTOR3 v3Trans, D3DXMATRIX matCamera, 
							float fAspect/*, float fZn, float fZf*/)
{
	if (!TheMeshIsCorrect)
	{
		return E_FAIL;
	}
	//World
	D3DXMatrixTranslation(&matW,v3Trans.x,v3Trans.y,v3Trans.z);
	//Projection
	D3DXMatrixPerspectiveFovLH(&matP, D3DX_PI / 4, fAspect, 10.0f, 1000.0f);
	//View
	D3DXMatrixInverse(&matV,NULL,&matCamera);
	//WorldView
	matWVP = matW*matV*matP;
	
	return S_OK;
}
