/************************************************************************/
/* 	Test application for Deferred Shading technology and it`s compatibility with current
/*	Chassis2 render technologies.	
/*	$id$
/*	File	: 					cLight.cpp
/*	Version	:					1.0
/*	Descript:					The light working class header.
/*	Autor	:					Morock
/*	Created :					2/26/2007
/************************************************************************/

#include "cLight.h"

CLight::CLight()
{
	v4Direction = D3DXVECTOR4(0.0f,0.0f,0.0f,0.0f);
	v4Position = D3DXVECTOR4(0.0f,0.0f,0.0f,0.0f);
	v4Ambient = D3DXVECTOR4(1.0f,1.0f,1.0f,0.0f);
	fRadius = 10.0f;
	iType = POINT_LIGHT;
	TheLightIsCorrect = FALSE;
}

CLight::~CLight()
{
		TheLightIsCorrect = FALSE;
}

HRESULT CLight::Load_Light(LPDIRECT3DDEVICE9 pd3dDevice, WCHAR *pchFileName)
{
	FILE *fLightFile;
	float pfTemp[11];
	
	//Open light file
	int itmp = 0;
	if (0 != _waccess_s(pchFileName,itmp))
	{
		return D3DERR_NOTFOUND;
	}
	errno_t err=_wfopen_s(&fLightFile,pchFileName,L"r");
	if (err!=0)
	{
		return D3DERR_NOTFOUND;
	}

	//Parse light file
	WCHAR pchStr[MAX_PATH];
	fseek(fLightFile,0,SEEK_SET);
	int iCount=0;
	do
	{
		if (fgetws(pchStr,MAX_PATH,fLightFile)!=NULL)
		{
			if ((pchStr[0]!='!')&&(pchStr[0]!='\n'))
			{
				pfTemp[iCount] = (float)_wtof(pchStr);
				iCount++;
			}
		}
	}while (iCount<11);
	//Color
	v4Ambient.x = pfTemp[0];
	v4Ambient.y = pfTemp[1];
	v4Ambient.z = pfTemp[2];

	//Position
	v4Position.x = pfTemp[3];
	//z in max = y in dx9?
	v4Position.z = pfTemp[4];
	v4Position.y = pfTemp[5];

	//Direction
	v4Direction.x = pfTemp[6];
	v4Direction.y = pfTemp[7];
	v4Direction.z = pfTemp[8];

	//Radius of attenuation
	fRadius = pfTemp[9];

	//Type of light
	iType = (int)pfTemp[10];
	
	fclose(fLightFile);

	TheLightIsCorrect = TRUE;

	return S_OK;
}
#pragma warning(disable : 4238)
HRESULT CLight::Update_Light(LPDIRECT3DDEVICE9 pd3dDevice,  D3DXVECTOR3 v3Trans,  
							 D3DXMATRIX matCamera, float fAspect, double fTime)
{
	if (!TheLightIsCorrect)
	{
		return S_FALSE;
	}
	
	//Matrix computing
	//World
	D3DXMatrixTranslation(&matW,v3Trans.x,v3Trans.y,v3Trans.z);
	//Projection
	D3DXMatrixPerspectiveFovLH(&matP, D3DX_PI / 4, fAspect, 10.0f, 1000.0f);
	//View
	D3DXMatrixInverse(&matV,NULL,&matCamera);
	//WorldView
	matWV = matW * matV;

	D3DXVec3Transform(&v4PositionV,
					  &D3DXVECTOR3(v4Position.x,v4Position.y,v4Position.z),
					  &matV);
	D3DXVec3Transform(&v4DirectionV,
					  &D3DXVECTOR3(v4Direction.x,v4Direction.y,v4Direction.z),
					  &matV);
	
	//Effects computing
	float fRnd = float(fTime) - INT(fTime); //Random number generator :-)
	if (POINT_LIGHT_ANI1==iType)
	{
		fRadius = 25*cos(180*fRnd);
	}
	
	return S_OK;
}