/************************************************************************/
/* 	My little Loger	
/*	$id$
/*	File	: 					cLog.h
/*	Version	:					1.0
/*	Descript:					The header file for class, provided loging				
/*	Autor	:					Morock
/*	Created :					2/15/2007
/************************************************************************/


/*
ANNOTATION
This class provides logging to a window and file. There can be several instanses of the
class, and, thats why, several loggers.
The class is UNICODE.

Sample using:

INT WINAPI WinMain (HINSTANCE hinst, HINSTANCE, LPSTR, int iNCmdShow)
{
...
cLog * pLogger = new cLog;
pLogger->INIT(L"log.txt", hinst, iNCmdShow);
pLogger->print(L"I want to log this as OK status",LOG_OK);
pLogger->print(L"I want to log this as information",LOG_INFO);
pLogger->print(L"I want to log this as warning message",LOG_WARN);
pLogger->print(L"I want to log this as error",LOG_ERR);
pLogger->print(L"I want to log this as constant message",LOG_CONST);
pLogger->~cLog();
...
}

The result of this will be the next text in log file, and on the log window:

Log started: 16/2/2007
{16:33:44.640} I want to log this as OK status [OK]
{16:33:44.656} I want to log this as information
{16:33:44.656} [WARNING] I want to log this as warning message
{16:33:44.656} [ERROR] I want to log this as error
I want to log this as constant message
Logging finished 
*/
#pragma once

#include <stdio.h>
#include <wchar.h>
#include <windows.h>
#include <io.h>

#define LOG_OK			0
#define LOG_INFO		1
#define LOG_WARN		2
#define LOG_ERR			3
#define LOG_CONST		4

class cLog
{
public:
	//BEG - Class variables
	bool IsInit;
	//END - Class variables
	//Constructor
	cLog();
	//Destructor: cleans all 
	~cLog();
	//Initialization: opens file name is pchFileName in the working directory of your program
	//!The file always rewrites!
	//Creates window (name is pchFileName)
	//Note: If you don`t want to create a window set hinstance=NULL
	//Returns TRUE, if everything is OK, FALSE if false ;-)
	bool INIT (WCHAR pchFileName[MAX_PATH], HINSTANCE hinstance, int iCmdShow);
	//Out to logging the string pchStr
	//type can be LOG_OK	({time} pchStr [OK])
	//type can be LOG_INFO	({time} pchStr) 
	//type can be LOG_WARN	({time} [WARNING] pchStr)
	//type can be LOG_ERR	({time} [ERROR] pchStr)
	//type can be LOG_CONST	(pchStr)
	//Returns number of printed symbols or -1 if error 
	int print(WCHAR *pchStr, int type);
	
protected:
	//BEG - Class variables

	//END - Class variables

private:
	//BEG - Class variables
	FILE * fLogFile;	//Handler to log file to be created
	HWND hWndConsol;	//Handler to Loger window
	HWND hWndTextOut;   //Handler to Loger EditBox
	HINSTANCE hinst;	//Loger(?) instance
	wchar_t pchFileName [MAX_PATH]; //Logger name = logger window name = logger file name
	//END - Class variables
	//LRESULT CALLBACK cMainWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
};