/************************************************************************/
/* 	Test application for Deferred Shading technology and it`s compatibility with current
/*	Chassis2 render technologies.
/*	$id$
/*	File	: 					main.cpp
/*	Version	:					1.0
/*	Descript:					The main application
/*	Autor	:					Morock
/*	Created :					2/15/2007
/************************************************************************/

#include "dxstdafx.h"
#include <d3d9types.h>
#include "resource.h"

#include "CMeshX.h"
#include "cLight.h"
#include "cLog.h"


//BEG - Global variables
	
	CMeshX	* pRenderObj;							//Render meshes
	CLight	* pRenderLights;						//Lights
	int		iNumOfMesh=0;							//Number of meshes in scene
	int		iNumOfLight=0;							//Number of lights in scene

	cLog	* pLogger = new cLog;	//Logger
	BYTE g_bKey[256];				//Handler of key down\up messages for camera and other
	bool g_bIsDS = TRUE;			//TRUE if render with Deferred Shading Simulation
	bool g_bIsCL = FALSE;			//TRUE if render with clasic light model

	//Global transformation data (use this for all meshes!!!)
	D3DXVECTOR3 g_v3Trans;			//Translation vector for WORLD MATRIX computing
	D3DXMATRIX	g_matCamera;		//Camera matrix for VIEW matrix computing
	float		g_fAspect;			//Aspect ratio for PROJECTION matrix computing

	
	//Global shader data
	ID3DXEffect*            g_pEffectPrep = NULL;	//HLSL effect for preparing of maps for DefShad
	D3DXHANDLE				hCurrentShadTech;		//Current shader tech. Using for switch the shaders tech.

	//Values for FPS counting, computing and printing
	float		fMaxFPS = -1;
	float		fMinFPS = 100000;
	LPD3DXFONT	pd3dxFont;							//Font interface to print FPS on render
	RECT * pRect = new RECT;						//Rectangle to draw the font for FPS

	//Render target textures
	LPDIRECT3DTEXTURE9			g_pRTBump		= NULL;  //Bump texture
	LPDIRECT3DTEXTURE9			g_pRTDiffuse	= NULL;  //Diffuse
	LPDIRECT3DTEXTURE9			g_pRTPos		= NULL;  //Position
	LPDIRECT3DTEXTURE9			g_pRTSpec		= NULL;  //Specular

	//Virtual screen declarations
	struct VIRTUALSCREEN_VERTEX
	{
		D3DXVECTOR3 p;		// Position
		D3DXVECTOR2 UV;		// The texture coordinates (for texture of dynamic process)
	}; 
	#define D3DFVF_VIRTUALSCREEN_VERTEX (D3DFVF_XYZ|D3DFVF_TEX1)
	VIRTUALSCREEN_VERTEX vsv[4];

	D3DXVECTOR2 g_ScreenSize;// size of screen in pixels

//END - Global variables

//BEG - Global functions
	void LogPrint(WCHAR *pchStr, int type); //Debug Log OUt
	
	//Key events handling with time and speed 
	bool Key_Update(	float fSpeed,			//   
						float fAngularSpeed,	//   
						float TickTime);		//     

	//Virtual Screen Vertex Buffer Init
	void CreateVirtualScreen ();

//END - Global functions

//--------------------------------------------------------------------------------------
// Rejects any devices that aren't acceptable by returning false
//--------------------------------------------------------------------------------------
bool CALLBACK IsDeviceAcceptable( D3DCAPS9* pCaps, D3DFORMAT AdapterFormat, 
                                  D3DFORMAT BackBufferFormat, bool bWindowed, void* pUserContext )
{
    // Typically want to skip backbuffer formats that don't support alpha blending
    IDirect3D9* pD3D = DXUTGetD3DObject(); 
    if( FAILED( pD3D->CheckDeviceFormat( pCaps->AdapterOrdinal, pCaps->DeviceType,
                    AdapterFormat, D3DUSAGE_QUERY_POSTPIXELSHADER_BLENDING, 
                    D3DRTYPE_TEXTURE, BackBufferFormat ) ) )
        return false;

    return true;
}


//--------------------------------------------------------------------------------------
// Before a device is created, modify the device settings as needed
//--------------------------------------------------------------------------------------
bool CALLBACK ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings, const D3DCAPS9* pCaps, void* pUserContext )
{
	return true;
}


//--------------------------------------------------------------------------------------
// Create any D3DPOOL_MANAGED resources here 
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnCreateDevice( IDirect3DDevice9* pd3dDevice, const D3DSURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
    HRESULT hr=S_OK;
	WCHAR	pStrLog[MAX_PATH];
	FILE	*fSceneFile;

//Load scene (TODO LATER)
	/*errno_t err= _wfopen_s(&fSceneFile,L"test.scene",L"r");
	if (err!=0)
	{
		MessageBoxW(NULL,L"Can`t open scene file!",L"DefShadSim Error!",MB_OK|MB_ICONERROR);
		return D3DERR_NOTFOUND;
	}*/


//Load lights
	WCHAR	pchLightFileName[MAX_PATH];

	errno_t err= _wfopen_s(&fSceneFile,L"test.lights",L"r");
	if (err!=0)
	{
		LogPrint(L"Can`t open scene lights file!",LOG_ERR);
		MessageBoxW(NULL,L"Can`t open scene lights file!",L"DefShadSim Error!",MB_OK|MB_ICONERROR);
		return D3DERR_NOTFOUND;
	}
	//Number of lights in .lights file
	while (fgetws(pchLightFileName,MAX_PATH,fSceneFile)!=NULL)
	{
		if ((pchLightFileName[0]!='!')&&(pchLightFileName[0]!='\n'))
		{
			iNumOfLight++;
		}
	}
	if (iNumOfLight>0)
	{
		fseek(fSceneFile,0,SEEK_SET);
		pRenderLights = new CLight[iNumOfLight];
		for (int i=0;i<iNumOfLight;i++)
		{
			if (NULL==fgetws(pchLightFileName,MAX_PATH,fSceneFile)) return S_FALSE;
			if (('!'==pchLightFileName[0])||('\n'==pchLightFileName[0]))
			{
				i--;
				continue;
			}
			wchar_t * chTemp = wcschr(pchLightFileName,L'\n');
			if (NULL!=chTemp)
			{
				chTemp[0] = 0;
			}
			//Create message to log about mesh loading
			wcscpy_s(pStrLog,sizeof(L"Load light: "),L"Load light: ");
			wcscat_s(pStrLog,pchLightFileName);
			LogPrint(pStrLog,LOG_INFO);
			hr = pRenderLights[i].Load_Light(pd3dDevice, pchLightFileName);
			if (FAILED(hr))
			{		
				MessageBoxW(NULL,L"Wrong scene!",L"DefShadSim Error!",MB_OK|MB_ICONERROR);
				LogPrint((WCHAR*)DXGetErrorString9(hr),LOG_ERR);
				return hr;
			}
			else
				LogPrint(L"     ",LOG_OK);
		}
	}	
	//pRenderLights
	fclose(fSceneFile);

//Load meshes
	WCHAR	pchMeshFileName[MAX_PATH];
	
	//Parse .mesh file
	//FILE	*fSceneFile;
	iNumOfMesh=0;
	err= _wfopen_s(&fSceneFile,L"test.meshs",L"r");
	if (err!=0)
	{
		LogPrint(L"Can`t open scene meshs file!",LOG_ERR);
		MessageBoxW(NULL,L"Can`t open scene meshs file!",L"DefShadSim Error!",MB_OK|MB_ICONERROR);
		return D3DERR_NOTFOUND;
	}
	//Number of meshes in .meshes file
	while (fgetws(pchMeshFileName,MAX_PATH,fSceneFile)!=NULL)
	{
		if ((pchMeshFileName[0]!='!')&&(pchMeshFileName[0]!='\n'))
		{
			iNumOfMesh++;
		}
	}
	if (iNumOfMesh<1)
	{
		LogPrint(L"No meshes in scene!",LOG_ERR);
		MessageBoxW(NULL,L"No meshes in scene!",L"DefShadSim Error!",MB_OK|MB_ICONERROR);
		return D3DERR_INVALIDCALL;
	}
	fseek(fSceneFile,0,SEEK_SET);
	
	//New mesh class exemplars
	pRenderObj = new CMeshX[iNumOfMesh];
	//Loading meshs from file
	for (int i=0; i<iNumOfMesh; i++)
	{
		if (NULL==fgetws(pchMeshFileName,MAX_PATH,fSceneFile)) return S_FALSE;
		if (('!'==pchMeshFileName[0])||('\n'==pchMeshFileName[0]))
		{
			i--;
			continue;
		}
		wchar_t * chTemp = wcschr(pchMeshFileName,L'\n');
		if (NULL!=chTemp)
		{
			chTemp[0] = 0;
		}
		//Create message to log about mesh loading
		wcscpy_s(pStrLog,sizeof(L"Load Mesh: "),L"Load Mesh: ");
		wcscat_s(pStrLog,pchMeshFileName);
		LogPrint(pStrLog,LOG_INFO);
		hr=pRenderObj[i].Load_Mesh(pd3dDevice,pchMeshFileName);	//loading...
		if (FAILED(hr))
		{		
			MessageBoxW(NULL,L"Wrong scene!",L"DefShadSim Error!",MB_OK|MB_ICONERROR);
			LogPrint((WCHAR*)DXGetErrorString9(hr),LOG_ERR);
			return hr;
		}
		else
			LogPrint(L"     ",LOG_OK);
	}
	fclose(fSceneFile);

	//Initializing view point
	float f_EyePoint_Z=-100.0f;
	float f_EyePoint_Y=0.0f;
	float f_EyePoint_X=0.0f;
	D3DXVECTOR3 vEyePt ( f_EyePoint_X, f_EyePoint_Y, f_EyePoint_Z );
	D3DXVECTOR3 vLookatPt( 0.0f, 0.0f, 1000.0f );
	D3DXVECTOR3 vUpVec ( 0.0f, 1.0f, 0.0f );
	D3DXMATRIX	matView;
	D3DXMatrixLookAtLH(&matView, &vEyePt, &vLookatPt, &vUpVec);
	D3DXMatrixInverse(&g_matCamera,NULL,&matView);
	g_v3Trans = D3DXVECTOR3(0,0,0);
	g_fAspect = 1.0f;
	for (int i=0; i<iNumOfMesh; i++)
		pRenderObj[i].Update_Mesh(pd3dDevice,g_v3Trans, g_matCamera, g_fAspect);
	for (int i=0; i<iNumOfLight; i++)
		pRenderLights[i].Update_Light(pd3dDevice,g_v3Trans, g_matCamera, g_fAspect, 0.0);

	return S_OK;
}

//--------------------------------------------------------------------------------------
// Release resources created in the OnCreateDevice callback here
//--------------------------------------------------------------------------------------
void CALLBACK OnDestroyDevice( void* pUserContext )
{
	LogPrint(L"Destroy Device. Data deleting",LOG_INFO);
	SAFE_DELETE_ARRAY(pRenderLights);
	LogPrint(L"     Lights",LOG_OK);
	SAFE_DELETE_ARRAY(pRenderObj);
	LogPrint(L"     Objects",LOG_OK);
	return;
}

//--------------------------------------------------------------------------------------
// Create any D3DPOOL_DEFAULT resources here 
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnResetDevice( IDirect3DDevice9* pd3dDevice, 
                                const D3DSURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
	 HRESULT hr=S_OK;

//Load effect
	WCHAR	pStrLog[MAX_PATH];
	WCHAR	pchEffectFileName[MAX_PATH];
	//Create message to log about effect loading
	wcscpy_s(pchEffectFileName,sizeof(L"shaders\\DS_prep.fx"),L"shaders\\DS_prep.fx");
	wcscpy_s(pStrLog,sizeof(L"Load shader program: "),L"Load shader program: ");
	wcscat_s(pStrLog,pchEffectFileName);
	LogPrint(pStrLog,LOG_INFO);
	hr = D3DXCreateEffectFromFileW((LPDIRECT3DDEVICE9)pd3dDevice, pchEffectFileName,
		NULL, NULL, NULL, NULL, 
		(LPD3DXEFFECT*)&g_pEffectPrep, NULL);
	if (FAILED(hr))
	{		
		LogPrint((WCHAR*)DXGetErrorString9(hr),LOG_ERR);
		return hr;
	}
	else
		LogPrint(L"     ",LOG_OK);

// Create fullscreen render targets textures
	LogPrint(L"Creating Render Target textures",LOG_INFO);

	g_ScreenSize.x = (float)pBackBufferSurfaceDesc->Width;
	g_ScreenSize.y = (float)pBackBufferSurfaceDesc->Height;

	//Bump
	hr =D3DXCreateTexture( pd3dDevice, 
		pBackBufferSurfaceDesc->Width,pBackBufferSurfaceDesc->Height,
		1, 
		D3DUSAGE_RENDERTARGET, 
		D3DFMT_A8R8G8B8, 
		D3DPOOL_DEFAULT, 
		&g_pRTBump);
	if (FAILED(hr))
	{		
		LogPrint((WCHAR*)DXGetErrorString9(hr),LOG_ERR);
		return hr;
	}
	else
		LogPrint(L"   Bump",LOG_OK);
	//Diffuse
	hr =D3DXCreateTexture( pd3dDevice, 
		pBackBufferSurfaceDesc->Width,pBackBufferSurfaceDesc->Height,
		1, 
		D3DUSAGE_RENDERTARGET, 
		D3DFMT_A8R8G8B8, 
		D3DPOOL_DEFAULT, 
		&g_pRTDiffuse);
	if (FAILED(hr))
	{		
		LogPrint((WCHAR*)DXGetErrorString9(hr),LOG_ERR);
		return hr;
	}
	else
		LogPrint(L"   Diffuse",LOG_OK);
	//Position
	hr =D3DXCreateTexture( pd3dDevice, 
		pBackBufferSurfaceDesc->Width,pBackBufferSurfaceDesc->Height,
		1, 
		D3DUSAGE_RENDERTARGET, 
		D3DFMT_A8B8G8R8,
		D3DPOOL_DEFAULT, 
		&g_pRTPos);
	if (FAILED(hr))
	{		
		LogPrint((WCHAR*)DXGetErrorString9(hr),LOG_ERR);
		return hr;
	}
	else
		LogPrint(L"   Position",LOG_OK);
	//Specular
	hr =D3DXCreateTexture( pd3dDevice, 
		pBackBufferSurfaceDesc->Width,pBackBufferSurfaceDesc->Height,
		1, 
		D3DUSAGE_RENDERTARGET, 
		D3DFMT_A8R8G8B8, 
		D3DPOOL_DEFAULT, 
		&g_pRTSpec);
	if (FAILED(hr))
	{		
		LogPrint((WCHAR*)DXGetErrorString9(hr),LOG_ERR);
		return hr;
	}
	else
		LogPrint(L"   Specular",LOG_OK);
	
	//Virtual Screen Vertex Buffer Init
	CreateVirtualScreen ();
	LogPrint(L"Virtual Screen Created",LOG_OK);

	//Create font interface for FPS printing while render
	hr = D3DXCreateFont(pd3dDevice,30,30,0,0,FALSE,0,OUT_TT_ONLY_PRECIS,0,0,L"Arial",&pd3dxFont);

	if (FAILED(hr))
	{
		LogPrint(L"Font creating",LOG_ERR);
	}
	else
		LogPrint(L"Font creating",LOG_OK);
	pRect->top = 0;
	pRect->left = 0;
	pRect->right = 68;
	pRect->bottom = 30;

	return S_OK;
}


//--------------------------------------------------------------------------------------
// Release resources created in the OnResetDevice callback here 
//--------------------------------------------------------------------------------------
void CALLBACK OnLostDevice( void* pUserContext )
{
	LogPrint(L"Device Lost. Releasing",LOG_INFO);
	SAFE_RELEASE(pd3dxFont);
	LogPrint(L"     Font",LOG_OK);
	SAFE_RELEASE(g_pEffectPrep);
	LogPrint(L"     Effect",LOG_OK);
	SAFE_RELEASE(g_pRTBump);
	SAFE_RELEASE(g_pRTDiffuse);
	SAFE_RELEASE(g_pRTPos);
	SAFE_RELEASE(g_pRTSpec);
	LogPrint(L"     Render targets",LOG_OK);
	return;
}



//--------------------------------------------------------------------------------------
// Handle updates to the scene
//--------------------------------------------------------------------------------------
void CALLBACK OnFrameMove( IDirect3DDevice9* pd3dDevice, double fTime, float fElapsedTime, void* pUserContext )
{
	Key_Update(40.0f,1.5f,fElapsedTime);
	//Update mesh position, view, and so on
	for (int i=0; i<iNumOfMesh; i++)
		pRenderObj[i].Update_Mesh(pd3dDevice,g_v3Trans, g_matCamera, g_fAspect);
	for (int i=0; i<iNumOfLight; i++)
		pRenderLights[i].Update_Light(pd3dDevice,g_v3Trans, g_matCamera, g_fAspect, fTime);
}


//--------------------------------------------------------------------------------------
// Render the scene 
//--------------------------------------------------------------------------------------
void CALLBACK OnFrameRender( IDirect3DDevice9* pd3dDevice, double fTime, float fElapsedTime, void* pUserContext )
{
    HRESULT hr;
	
	IDirect3DSurface9 * pBackBuffer;
	LPDIRECT3DSURFACE9 g_pFullScreenTextureSurf=NULL;
	
	//Render with Deferred Shading Simulation
	//Render all tech to textures, and then render light by texture information
	if (g_bIsDS)
	{
		UINT iPasses;

	//Render to Diffuse Texture
		hCurrentShadTech = "RenderDiffuseToTexture";
		// Clear the render target and the zbuffer 
		hr = pd3dDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_ARGB(0, 0, 0, 0), 1.0f, 0);
		if( SUCCEEDED( pd3dDevice->BeginScene() ) )
		{
			//Draw meshes
			for (int i=0; i<iNumOfMesh; i++)
			{
				hr = pRenderObj[i].Draw_Mesh(pd3dDevice, g_pEffectPrep, hCurrentShadTech);
				if (FAILED(hr))
				{
					LogPrint(L"Mesh Render Error",LOG_ERR);
				}
			}
			hr = pd3dDevice->EndScene();
		}
		//Getting render target picture
		pd3dDevice->GetBackBuffer(0, 0, D3DBACKBUFFER_TYPE_MONO, &pBackBuffer);
		g_pRTDiffuse->GetSurfaceLevel(0, &g_pFullScreenTextureSurf);
		//     surface     2-
		pd3dDevice->StretchRect( pBackBuffer, NULL, g_pFullScreenTextureSurf, NULL, D3DTEXF_LINEAR );
		SAFE_RELEASE( pBackBuffer );
		SAFE_RELEASE(g_pFullScreenTextureSurf);

	//Render to Bump Texture
		hCurrentShadTech = "RenderNormalToTexture";
		// Clear the render target and the zbuffer 
		hr = pd3dDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_ARGB(0, 0, 0, 0), 1.0f, 0);
		if( SUCCEEDED( pd3dDevice->BeginScene() ) )
		{
			//Draw meshes
			for (int i=0; i<iNumOfMesh; i++)
			{
				hr = pRenderObj[i].Draw_Mesh(pd3dDevice, g_pEffectPrep, hCurrentShadTech);
				if (FAILED(hr))
				{
					LogPrint(L"Mesh Render Error",LOG_ERR);
				}
			}
			hr = pd3dDevice->EndScene();
		}
		//Getting render target picture
		pd3dDevice->GetBackBuffer(0, 0, D3DBACKBUFFER_TYPE_MONO, &pBackBuffer);
		g_pRTBump->GetSurfaceLevel(0, &g_pFullScreenTextureSurf);
		//     surface     2-
		pd3dDevice->StretchRect( pBackBuffer, NULL, g_pFullScreenTextureSurf, NULL, D3DTEXF_LINEAR );
		SAFE_RELEASE( pBackBuffer );
		SAFE_RELEASE(g_pFullScreenTextureSurf);

	//Render to Spec Texture
		hCurrentShadTech = "RenderSpecularToTexture";
		// Clear the render target and the zbuffer 
		hr = pd3dDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_ARGB(0, 0, 0, 0), 1.0f, 0);
		if( SUCCEEDED( pd3dDevice->BeginScene() ) )
		{
			//Draw meshes
			for (int i=0; i<iNumOfMesh; i++)
			{
				hr = pRenderObj[i].Draw_Mesh(pd3dDevice, g_pEffectPrep, hCurrentShadTech);
				if (FAILED(hr))
				{
					LogPrint(L"Mesh Render Error",LOG_ERR);
				}
			}
			hr = pd3dDevice->EndScene();
		}
		//Getting render target picture
		pd3dDevice->GetBackBuffer(0, 0, D3DBACKBUFFER_TYPE_MONO, &pBackBuffer);
		g_pRTSpec->GetSurfaceLevel(0, &g_pFullScreenTextureSurf);
		//     surface     2-
		pd3dDevice->StretchRect( pBackBuffer, NULL, g_pFullScreenTextureSurf, NULL, D3DTEXF_LINEAR );
		SAFE_RELEASE( pBackBuffer );
		SAFE_RELEASE(g_pFullScreenTextureSurf);

		//Render to Position Texture
		hCurrentShadTech = "RenderPositionToTexture";
		// Clear the render target and the zbuffer 
		hr = pd3dDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_ARGB(0, 0, 0, 0), 1.0f, 0);
		if( SUCCEEDED( pd3dDevice->BeginScene() ) )
		{
			//Draw meshes
			for (int i=0; i<iNumOfMesh; i++)
			{
				hr = pRenderObj[i].Draw_Mesh(pd3dDevice, g_pEffectPrep, hCurrentShadTech);
				if (FAILED(hr))
				{
					LogPrint(L"Mesh Render Error",LOG_ERR);
				}
			}
			hr = pd3dDevice->EndScene();
		}
		//Getting render target picture
		pd3dDevice->GetBackBuffer(0, 0, D3DBACKBUFFER_TYPE_MONO, &pBackBuffer);
		g_pRTPos->GetSurfaceLevel(0, &g_pFullScreenTextureSurf);
		//     surface  , yes, you know already?
		pd3dDevice->StretchRect( pBackBuffer, NULL, g_pFullScreenTextureSurf, NULL, D3DTEXF_LINEAR );
		SAFE_RELEASE( pBackBuffer );
		SAFE_RELEASE(g_pFullScreenTextureSurf);

	//Render Lights
		pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET,0xff000000,0.0f, 0x0);
		if( SUCCEEDED( pd3dDevice->BeginScene() ) )
		{
			for (int i=0; i<iNumOfLight; i++)
			{
				
				if(i==0)
				{
					//Set Render States
					pd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
					pd3dDevice->SetRenderState(D3DRS_ALPHATESTENABLE, FALSE);
					pd3dDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
					pd3dDevice->SetRenderState(D3DRS_AMBIENT, 0x00000000);
					pd3dDevice->SetRenderState(D3DRS_SPECULARENABLE, FALSE);
					pd3dDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);	
					pd3dDevice->SetRenderState(D3DRS_ZENABLE, TRUE);
				}
				else
				{
					//Set Render States
					pd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
					pd3dDevice->SetRenderState(D3DRS_ALPHATESTENABLE, TRUE);
					pd3dDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
					pd3dDevice->SetRenderState(D3DRS_AMBIENT, 0x00000000);
					pd3dDevice->SetRenderState(D3DRS_SPECULARENABLE, FALSE);
					pd3dDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);	
					pd3dDevice->SetRenderState(D3DRS_ZENABLE, FALSE);
					pd3dDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
					pd3dDevice->SetRenderState(D3DRS_DESTBLEND,D3DBLEND_ONE);
				}
				//Switching technique by light type
				switch(pRenderLights[i].iType)
				{
				case 1:
					{
						hr = g_pEffectPrep->SetTechnique("RenderDSSun");
					}
					break;
				case 2:
					{
						hr = g_pEffectPrep->SetTechnique("RenderDSPoint");
					}
					break;
				case 21:
					{
						hr = g_pEffectPrep->SetTechnique("RenderDSPoint");
					}
					break;
				default:
					{
						continue;
					}					
				}
							
				//Set shaders properties
				//Matrices
				D3DXMATRIX Ident, matW, matV, matP, matWV, matWVP;
				D3DXMatrixIdentity(&Ident);
				//World
				D3DXMatrixTranslation(&matW,0,0,0);
				//Projection
				D3DXMatrixPerspectiveFovLH(&matP, D3DX_PI / 4, g_fAspect, 10.0f, 1000.0f);
				//View
				//Initializing view point
				float f_EyePoint_Z=-100.0f;
				float f_EyePoint_Y=0.0f;
				float f_EyePoint_X=0.0f;
				D3DXVECTOR3 vEyePt ( f_EyePoint_X, f_EyePoint_Y, f_EyePoint_Z );
				D3DXVECTOR3 vLookatPt( 0.0f, 0.0f, 1000.0f );
				D3DXVECTOR3 vUpVec ( 0.0f, 1.0f, 0.0f );
				D3DXMatrixLookAtLH(&matV, &vEyePt, &vLookatPt, &vUpVec);

				//WorldView
				matWV = matW * matV;
				hr = g_pEffectPrep->SetMatrix("matWV",&matWV);
				hr = g_pEffectPrep->SetMatrix("matW",&matW);
				hr = g_pEffectPrep->SetMatrix("matV",&matV);
				hr = g_pEffectPrep->SetMatrix("matP",&matP);
				hr = g_pEffectPrep->SetMatrix("matWVP",&Ident);

				//And Inverted matrices as in NV demo:
				D3DXMATRIX matInvP, matInvV, matInvVP, matInvTransW, matVP;
				//World Inverse Transpose
				D3DXMatrixInverse(&matInvTransW, NULL, &matW);
				D3DXMatrixTranspose(&matInvTransW,&matInvTransW);
				hr = g_pEffectPrep->SetMatrix("matInvTrW",&matInvTransW);
				//View Inverse
				D3DXMatrixInverse(&matInvV, NULL, &matV);
				hr = g_pEffectPrep->SetMatrix("matInvV",&matInvV);
				//Projection Inverse
				D3DXMatrixInverse(&matInvP, NULL, &matP);
				hr = g_pEffectPrep->SetMatrix("matInvP",&matInvP);
				//View Projection Inverse
				matVP = matV*matP;
				D3DXMatrixInverse(&matInvVP, NULL, &matVP);
				hr = g_pEffectPrep->SetMatrix("matInvVP",&matInvVP);
				
				//Render textures
				hr = g_pEffectPrep->SetTexture("m_TexAlfa",g_pRTPos);//texture m_TexAlfa - position :-);
				hr = g_pEffectPrep->SetTexture("m_TexBump",g_pRTBump);//texture m_TexBump;
				hr = g_pEffectPrep->SetTexture("m_TexDiffuse",g_pRTDiffuse);//texture m_TexDiffuse;
				hr = g_pEffectPrep->SetTexture("m_TexSpecular",g_pRTSpec);//texture m_TexSpecular;
				//Light properties
				hr = g_pEffectPrep->SetVector("l_colAmbient",&pRenderLights[i].v4Ambient);
				hr = g_pEffectPrep->SetVector("l_dir",&pRenderLights[i].v4DirectionV);
				hr = g_pEffectPrep->SetVector("l_pos",&pRenderLights[i].v4PositionV);
				hr = g_pEffectPrep->SetFloat("l_radius",pRenderLights[i].fRadius);
				//Others
				
				//Drawing
				g_pEffectPrep->Begin(&iPasses,0);
					g_pEffectPrep->BeginPass(0);			
						pd3dDevice->SetFVF( D3DFVF_VIRTUALSCREEN_VERTEX );			
						pd3dDevice->DrawPrimitiveUP(D3DPT_TRIANGLEFAN,2,vsv,sizeof(VIRTUALSCREEN_VERTEX));
					g_pEffectPrep->EndPass();
				g_pEffectPrep->End();
			}

			//FPS
			float fCurFPS = DXUTGetFPS();
			if (fCurFPS>fMaxFPS)
			{
				fMaxFPS = fCurFPS;
			}
			else
				if ((fCurFPS<fMinFPS)&&(fCurFPS>1))
				{
					fMinFPS = fCurFPS;
				}
				//FPS to screen
				WCHAR pchStr[5];
				_ltow_s ((long)fCurFPS,pchStr,5,10);
				pd3dxFont->DrawTextW(NULL,pchStr,2,pRect, DT_LEFT, D3DCOLOR_ARGB(100,256,256,256));
			
			hr = pd3dDevice->EndScene();
		}
		return;
	}
	
//Render without DS
	// Clear the render target and the zbuffer 
    V( pd3dDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_ARGB(0, 0, 0, 0), 1.0f, 0) );
	if( SUCCEEDED( pd3dDevice->BeginScene() ) )
	{
		for (int j=0; j<iNumOfLight; j++)
		{
			//Turning on blending
			pd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
			pd3dDevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
			pd3dDevice->SetRenderState(D3DRS_DESTBLEND,D3DBLEND_ONE);
	
			//Switching technique by light type
			if (g_bIsCL)
			switch(pRenderLights[j].iType)
			{
			case 1:
				{
					hCurrentShadTech = "RenderClassicSun";
				}
				break;
			case 2:
				{
					hCurrentShadTech = "RenderClassicPoint";
				}
				break;
			case 21:
				{
					hCurrentShadTech = "RenderClassicPoint";
				}
				break;
			default:
				{
					continue;
				}
			}

			g_pEffectPrep->SetVector("l_colAmbient",&pRenderLights[j].v4Ambient);
			g_pEffectPrep->SetVector("l_dir",&pRenderLights[j].v4DirectionV);
			g_pEffectPrep->SetVector("l_pos",&pRenderLights[j].v4PositionV);
			g_pEffectPrep->SetFloat("l_radius",pRenderLights[j].fRadius);

				//Draw meshes
				for (int i=0; i<iNumOfMesh; i++)
				{
					hr = pRenderObj[i].Draw_Mesh(pd3dDevice, g_pEffectPrep, hCurrentShadTech);
					if (FAILED(hr))
					{
						LogPrint(L"Mesh Render Error",LOG_ERR);
					}
				}
		}
		
		//FPS
		float fCurFPS = DXUTGetFPS();
		if (fCurFPS>fMaxFPS)
		{
			fMaxFPS = fCurFPS;
		}
		else
			if ((fCurFPS<fMinFPS)&&(fCurFPS>1))
			{
				fMinFPS = fCurFPS;
			}
		//FPS to screen
		WCHAR pchStr[5];
		_ltow_s ((LONG)fCurFPS,pchStr,5,10);
		pd3dxFont->DrawTextW(NULL,pchStr,2,pRect, DT_LEFT, D3DCOLOR_ARGB(100,256,256,256));

		hr = pd3dDevice->EndScene();
	}

}


//--------------------------------------------------------------------------------------
// Handle messages to the application 
//--------------------------------------------------------------------------------------
LRESULT CALLBACK MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, 
                          bool* pbNoFurtherProcessing, void* pUserContext )
{
	switch (uMsg)
	{

	case WM_KEYDOWN: g_bKey[wParam] = 1;	break;
	case WM_KEYUP:	 g_bKey[wParam] = 0;	break;

	case WM_MOUSEMOVE:
		break;
	case WM_SIZE:
		{
			//LogPrint(L"Resizing window",LOG_INFO);
			WORD ww,hh;
			ww = LOWORD(lParam);
			hh = HIWORD(lParam);
			if(0==hh)
				hh=1;
			g_fAspect = (float)ww/(float)hh;
		}
	}
	return 0;
}


//--------------------------------------------------------------------------------------
// Initialize everything and go into a render loop
//--------------------------------------------------------------------------------------
INT WINAPI WinMain( HINSTANCE hinst, HINSTANCE, LPSTR, int iNCmdShow)
{
    // Enable run-time memory check for debug builds.
#if defined(DEBUG) | defined(_DEBUG)
    _CrtSetDbgFlag( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
#endif

    // Set the callback functions
    DXUTSetCallbackDeviceCreated( OnCreateDevice );
    DXUTSetCallbackDeviceReset( OnResetDevice );
    DXUTSetCallbackDeviceLost( OnLostDevice );
    DXUTSetCallbackDeviceDestroyed( OnDestroyDevice );
    DXUTSetCallbackMsgProc( MsgProc );
    DXUTSetCallbackFrameRender( OnFrameRender );
    DXUTSetCallbackFrameMove( OnFrameMove );
   
    // TODO: Perform any application-level initialization here
	//Creating Logger
#if defined(DEBUG) | defined(_DEBUG)
	pLogger->INIT(L"log.txt", NULL, iNCmdShow);
#endif

    // Initialize DXUT and create the desired Win32 window and Direct3D device for the application
    DXUTInit( true, true, true ); // Parse the command line, handle the default hotkeys, and show msgboxes
    DXUTSetCursorSettings( true, true ); // Show the cursor and clip it when in full screen
	LogPrint(L"DXUTInit",LOG_OK);
	DXUTCreateWindow( L"Deferred Shading Simulation" );
	LogPrint(L"DXUTCreateWindow",LOG_OK);
	DXUTCreateDevice( D3DADAPTER_DEFAULT, true, 640, 480, IsDeviceAcceptable, ModifyDeviceSettings );
	LogPrint(L"DXUTCreateDevice",LOG_OK);

    // Start the render loop
    DXUTMainLoop();
	LogPrint(L"DXUtil main loop stopped",LOG_INFO);

    // TODO: Perform any application-level cleanup here
	//Kill POOL recourses
	LogPrint(L"Releasing:",LOG_INFO);
	SAFE_RELEASE(pd3dxFont);
	LogPrint(L"     Font",LOG_OK);
	SAFE_RELEASE(g_pEffectPrep);
	LogPrint(L"     Effect",LOG_OK);
	SAFE_RELEASE(g_pRTBump);
	SAFE_RELEASE(g_pRTDiffuse);
	SAFE_RELEASE(g_pRTPos);
	SAFE_RELEASE(g_pRTSpec);
	LogPrint(L"     Render targets",LOG_OK);
	//Kill managed recourses
	LogPrint(L"Data deleting:",LOG_INFO);
	SAFE_DELETE_ARRAY(pRenderLights);
	LogPrint(L"     Lights",LOG_OK);
	SAFE_DELETE_ARRAY(pRenderObj);
	LogPrint(L"     Objects",LOG_OK);
	
	//Log the Max\Min FPS values
	WCHAR pStrLog[MAX_PATH], pchTemp[MAX_PATH];
	wcscpy_s(pStrLog,sizeof(L"Max FPS: "),L"Max FPS: ");
	_ltow_s ((LONG)fMaxFPS,pchTemp,MAX_PATH,10);
	wcscat_s(pStrLog, pchTemp);
	wcscat_s(pStrLog, L". Min FPS: ");
	_ltow_s ((LONG)fMinFPS,pchTemp,MAX_PATH,10);
	wcscat_s(pStrLog, pchTemp);
	LogPrint(pStrLog,LOG_INFO);
	//Kill logger
	SAFE_DELETE(pLogger);

    return DXUTGetExitCode();
}


//-----------------------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------------------
//My functions
//-----------------------------------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------------------------------


//-----------------------------------------------------------------------------------------------------------
//Debug Log OUt
//-----------------------------------------------------------------------------------------------------------
void LogPrint(WCHAR *pchStr, int type)
{
#if defined(DEBUG) | defined(_DEBUG)
	if (NULL==pLogger)
	{
		return;
	}
	pLogger->print(pchStr,type);
#endif
}


//-----------------------------------------------------------------------------------------------------------
//Key events handling with time and speed 
//-----------------------------------------------------------------------------------------------------------
bool Key_Update(	float fSpeed,			//   
					float fAngularSpeed,	//   
					float TickTime)			//     
{

	D3DXVECTOR3 vecT(0.0f, 0.0f, 0.0f);
	D3DXVECTOR3 vecR(0.0f, 0.0f, 0.0f);
	D3DXMATRIX	matT, matR;
	D3DXQUATERNION qR;
	bool		bChange=false;

	vecT.x=vecT.y=vecT.z=0.0f;
	vecR.x=vecR.y=vecR.z=0.0f;

	bChange=false;

	{
		//Turning keys
		if(g_bKey[VK_RIGHT])		{bChange=true;vecR.x =- 1.0f;} // Turn Right
		if(g_bKey[VK_LEFT])			{bChange=true;vecR.x =+ 1.0f;} // Turn Left
		if(g_bKey[VK_UP])			{bChange=true;vecR.y =- 1.0f;} // Pitch Down
		if(g_bKey[VK_DOWN])			{bChange=true;vecR.y =+ 1.0f;} // Pitch Up
		if(g_bKey['E'])				{bChange=true;vecR.z =+ 1.0f;} // Turn Right
		if(g_bKey['Q'])				{bChange=true;vecR.z =- 1.0f;} // Turn Left
		//Moving keys
		if(g_bKey['A'])				{bChange=true;vecT.x =+ 1.0f;} // Slide Left
		if(g_bKey['D'])				{bChange=true;vecT.x =- 1.0f;} // Slide Right
		if(g_bKey[VK_NEXT])			{bChange=true;vecT.y =+ 1.0f;} // Slide Down
		if(g_bKey[VK_PRIOR])		{bChange=true;vecT.y =- 1.0f;} // Slide Up
		if(g_bKey['W'])				{bChange=true;vecT.z =- 1.0f;} // Move Forward
		if(g_bKey['S'])				{bChange=true;vecT.z =+ 1.0f;} // Move Backward

		//Shader switching
		//No light render
		if(g_bKey['1'])				{hCurrentShadTech="RenderDiffuseToTexture";	g_bIsDS = FALSE;g_bIsCL = FALSE;}
		if(g_bKey['2'])				{hCurrentShadTech="RenderSpecularToTexture";g_bIsDS = FALSE;g_bIsCL = FALSE;}
		if(g_bKey['3'])				{hCurrentShadTech="RenderPositionToTexture";g_bIsDS = FALSE;g_bIsCL = FALSE;}
		if(g_bKey['4'])				{hCurrentShadTech="RenderNormalToTexture";	g_bIsDS = FALSE;g_bIsCL = FALSE;}
		if(g_bKey['5'])				{
										g_bIsCL = TRUE;	g_bIsDS = FALSE;	//Turning on classic render
										LogPrint(L"Classic light render activated",LOG_INFO);
									}
		if(g_bKey['0'])				{
										g_bIsDS = TRUE;	g_bIsCL = FALSE;	//Turning on deferred shading
										LogPrint(L"Deferred Shading render activated",LOG_INFO);
									}
		
		vecT = -vecT;
		vecR = -vecR;
	}

	//
	// Update position and view matricies
	//
	if(bChange)
	{
		vecT = vecT * TickTime * fSpeed;
		vecR = vecR * TickTime * fAngularSpeed;

		D3DXMatrixTranslation(&matT, vecT.x, vecT.y, vecT.z);

		//matPOS = matT * matPOS;
		D3DXMatrixMultiply(&g_matCamera, &matT, &g_matCamera);

		D3DXQuaternionRotationYawPitchRoll(&qR, vecR.x, vecR.y, vecR.z);
		D3DXMatrixRotationQuaternion(&matR, &qR);

		// matPOS = matR * matPOS;
		D3DXMatrixMultiply(&g_matCamera, &matR, &g_matCamera);
	}
	return bChange;
}

//Virtual Screen Vertex Buffer Init
void CreateVirtualScreen ()
{
	const float ScreenQuad = 1.0f;//0.75f;
	vsv[0].p=D3DXVECTOR3(-ScreenQuad,-ScreenQuad, 1);
	vsv[3].p=D3DXVECTOR3( ScreenQuad,-ScreenQuad, 1);
	vsv[2].p=D3DXVECTOR3( ScreenQuad, ScreenQuad, 1);
	vsv[1].p=D3DXVECTOR3(-ScreenQuad, ScreenQuad, 1);
	vsv[0].UV=D3DXVECTOR2(0,1);
	vsv[3].UV=D3DXVECTOR2(1,1);
	vsv[2].UV=D3DXVECTOR2(1,0);
	vsv[1].UV=D3DXVECTOR2(0,0);
	return;
}